#
# Pcap.pm
#
# An interface to the LBL pcap(3) library.  This module simply
# bootstraps the extensions defined in Pcap.xs
#
# Copyright (c) 1999-2000 Tim Potter. All rights reserved. This program is
# free# software; you can redistribute it and/or modify it under the same
# terms as Perl itself.
#
# Comments/suggestions to tpot@frungy.org
#
# $Id: Pcap.pm,v 1.9 2000/05/15 05:02:50 tpot Exp $
#
# Patched for Win32 - JLM - 03/09/2004 v 0.04.02
#

package Net::Pcap;

require Exporter;
require DynaLoader;

use vars qw($VERSION @ISA);

@ISA = qw(Exporter DynaLoader);

our %EXPORT_TAGS = (
'mode' => ['MODE_CAPT', 'MODE_STAT', 'MODE_MON'],
'src' => ['SRC_FILE', 'SRC_IFLOCAL', 'SRC_IFREMOTE'],
'openflag' => ['OPENFLAG_PROMISCUOUS', 'OPENFLAG_DATATX_UDP', 'OPENFLAG_NOCAPTURE_RPCAP'],
'samp' => ['SAMP_NOSAMP', 'SAMP_1_EVERY_N', 'SAMP_FIRST_AFTER_N_MS'],
'rpcap' => ['RMTAUTH_NULL', 'RMTAUTH_PWD'],
);

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'mode'} },
                   @{ $EXPORT_TAGS{'src'} },
                   @{ $EXPORT_TAGS{'openflag'} },
                   @{ $EXPORT_TAGS{'samp'} },
                   @{ $EXPORT_TAGS{'rpcap'} },
);

our @EXPORT = qw(
);

@EXPORT = qw();

$VERSION = '0.04';

use constant SRC_FILE => 2;
use constant SRC_IFLOCAL => 3;
use constant SRC_IFREMOTE => 4;

use constant OPENFLAG_PROMISCUOUS => 1;
use constant OPENFLAG_DATATX_UDP => 2;
use constant OPENFLAG_NOCAPTURE_RPCAP => 4;

use constant SAMP_NOSAMP =>	0;
use constant SAMP_1_EVERY_N	=> 1;
use constant SAMP_FIRST_AFTER_N_MS => 2;

use constant RMTAUTH_NULL => 0;
use constant RMTAUTH_PWD => 1;

use constant MODE_CAPT => 0;
use constant MODE_STAT => 1;
use constant MODE_MON => 2;

bootstrap Net::Pcap $VERSION;

1;

# autoloaded methods go after the END token (&& pod) below

__END__

=head1 NAME

Net::Pcap - Interface to pcap(3) LBL packet capture library

[
B<Caution:> this is a patched version of Net::Pcap for Win32.
It's a beta version for this platform. Don't use it in a production script.
In agreement with Tim Potter, it will be incorporated in a future release
of Net::Pcap.

In order to use Net::Pcap in your Perl scripts, it is necessary to install 
WinPcap 3.1 beta 3 or WinPcap 3.0 on your machine; see 
L<http://winpcap.polito.it/install/default.htm>.

Home page for this version: L<http://www.bribes.org/perl/wnetpcap.html>

Report bug to Jean-Louis Morel: jl_morel@bribes.org . Thanks.
]

=head1 SYNOPSIS

  use Net::Pcap;

=head1 DESCRIPTION

B<Net::Pcap> is a Perl binding to the LBL pcap(3) library, version
0.4.  The README for libpcap describes itself as:

  "a system-independent interface for user-level packet capture.
  libpcap provides a portable framework for low-level network
  monitoring.  Applications include network statistics collection,
  security monitoring, network debugging, etc."

=head1 FUNCTIONS

All functions defined by B<Net::Pcap> are direct mappings to the
libpcap functions.  Consult the pcap(3) documentation and source code
for more information.

Arguments that change a parameter, for example B<Net::Pcap::lookupdev()>,
are passed that parameter as a reference.  This is to retain
compatibility with previous versions of B<Net::Pcap>.

=head2 Lookup functions

=over

=item B<Net::Pcap::lookupdev(\$err);>

Returns the name of a network device that can be used with
B<Net::Pcap::open_live()> function.  On error, the $err parameter is
filled with an appropriate error message else it is undefined.

=item B<Net::Pcap::findalldevs(\$err [,\%description]);>

Returns a list of all network device names that can be used with
B<Net::Pcap::open_live()> function. If the optional parameter is provided,
the hash C<%description> is filled with the description string of each
network device. On error, the $err parameter is filled with an appropriate
error message else it is undefined.

  my ($error, %description);
  foreach(Net::Pcap::findalldevs(\$error, \%description)) {
    print "$_\n   $description{$_}\n\n";
  }
  print $error if defined $error;

=item B<Net::Pcap::lookupnet($dev, \$net, \$mask, \$err);>

Determine the network number and netmask for the device specified in
$dev.  The function returns 0 on success and sets the $net and
$mask parameters with values.  On failure it returns -1 and the
$err parameter is filled with an appropriate error message.

=back

=head2 Packet capture functions

=over

=item Net::Pcap::open_live($dev, $snaplen, $promisc, $to_ms, \$err);

Returns a packet capture descriptor for looking at packets on the
network.  The $dev parameter specifies which network interface to
capture packets from.  The $snaplen and $promisc parameters specify
the maximum number of bytes to capture from each packet, and whether
to put the interface into promiscuous mode, respectively.  The $to_ms
parameter specifies a read timeout in ms.  The packet descriptor will
be undefined if an error occurs, and the $err parameter will be set
with an appropriate error message.

=item B<Net::Pcap::loop($pcap_t, $cnt, \&callback_fn, $user_data);>

Read $cnt packets from the packet capture descriptor $pcap_t and call
the perl function &callback_fn with an argument of $user_data.  If $cnt is negative, then the function loops forever or until an error occurs.

The callback function is also passed packet header information and
packet data like so:

  sub process_pkt {
      my($user_data, $hdr, $pkt) = @_;

      ...
  }

The header information is a reference to a hash containing the
following fields.

=over

=item * len

The total length of the packet.

=item * caplen

The actual captured length of the packet data.  This corresponds to
the snapshot length parameter passed to B<Net::Pcap::open_live()>.

=item * tv_sec

Seconds value of the packet timestamp.

=item * tv_usec

Microseconds value of the packet timestamp.

=back

=item B<Net::Pcap::open_offline($filename, \$err);>

Return a packet capture descriptor to read from a previously created
"savefile".  The returned descriptor is undefined if there was an
error and in this case the $err parameter will be filled.  Savefiles
are created using the Net::Pcap::dump_* commands.

=item B<Net::Pcap::close($pcap_t);>

Close the packet capture device associated with descriptor $pcap_t.

=item B<Net::Pcap::dispatch($pcap_t, $cnt, \&callback_fn, $user_data);>

Collect $cnt packets and process them with callback function
&callback_fn.  if $cnt is -1, all packets currently buffered are
processed.  If $cnt is 0, process all packets until an error occurs.

=item B<Net::Pcap::next($pcap_t, \%hdr);>

Return the next available packet on the interface associated with
packet descriptor $pcap_t.  Into the %hdr hash is stored the received
packet header.  If not packet is available, the return value and
header is undefined.

=item B<Net::Pcap::compile($pcap_t, \$filter_t, $filter_str, $optimize, $netmask);>

Compile the filter string contained in $filter_str and store it in
$filter_t.  A description of the filter language can be found in the
libpcap source code, or the manual page for tcpdump(8) .  The filter
is optimized the filter if the $optimize variable is true.  The
netmask of the network device must be specified in the $netmask
parameter.  The function returns 0 if the compilation was successful,
or -1 if there was a problem.

=item B<Net::Pcap::setfilter($pcap_t, $filter_t);>

Associate the compiled filter stored in $filter_t with the packet
capture descriptor $pcap_t.

=back

=head2 Savefile commands

=over

=item B<Net::Pcap::dump_open($pcap_t, $filename);>

Open a savefile for writing and return a descriptor for doing so.  If
$filename is "-" data is written to standard output.  On error, the
return value is undefined and B<Net::Pcap::geterr()> can be used to
retrieve the error text.

=item B<Net::Pcap::dump($pcap_dumper_t, \%hdr, $pkt);>

Dump the packet described by header %hdr and packet data $pkt to the
savefile associated with $pcap_dumper_t.  The packet header has the
same format as that passed to the B<Net::Pcap::loop()> callback.

=item B<Net::Pcap::dump_close($pcap_dumper_t);>

Close the savefile associated with descriptor $pcap_dumper_t.

=back

=head2 Status functions

=over

=item B<$error = Net::Pcap::setnonblock($pcap_t, $nonblock, \$err);>

puts a capture descriptor, opened with C<pcap_open_live()>,
into "non-blocking" mode, or takes it out of "non-blocking" mode, depending
on whether the $nonblock argument is non-zero or zero. It has no effect on
"savefiles". If there is an error, -1 is returned and $err is filled in
with an appropriate error message; otherwise, 0 is returned. In "non-
blocking" mode, an attempt to read from the capture descriptor with
C<pcap_dispatch()> will, if no packets are currently available to be read,
return 0 immediately rather than blocking waiting for packets to arrive.
C<pcap_loop()> and C<pcap_next()> will not work in "non-blocking" mode.

=item B<$nonblock = Net::Pcap::getnonblock($pcap_t, \$err);>

returns the current "non-blocking" state of the capture descriptor; it
always returns 0 on "savefiles". If there is an error, -1 is returned and
C<$err> is filled in with an appropriate error message.

=item B<Net::Pcap::datalink($pcap_t);>

Returns the link layer type associated with the currently open device.

=item B<Net::Pcap::snapshot($pcap_t);>

Returns the snapshot length (snaplen) specified in the call to
B<Net::Pcap::open_live()>.

=item B<Net::Pcap::is_swapped($pcap_t);>

This function returns true if the endianess of the currently open
savefile is different from the endianess of the machine.

=item B<Net::Pcap::major_version($pcap_t);>

Return the major version number of the pcap library used to write the
currently open savefile.

=item B<Net::Pcap::minor_version($pcap_t);>

Return the minor version of the pcap library used to write the
currently open savefile.

=item B<Net::Pcap::stats($pcap_t, \%stats);>

Returns a hash containing information about the status of packet
capture device $pcap_t.  The hash contains the following fields.

=over

=item * B<ps_recv>

The number of packets received by the packet capture software.

=item * B<ps_drop>

The number of packets dropped by the packet capture software.

=item * B<ps_ifdrop>

The number of packets dropped by the network interface.

=back

=item B<Net::Pcap::file($pcap_t);>

Return the filehandle associated with a savefile opened with
B<Net::Pcap::open_offline()>.

=item B<Net::Pcap::fileno($pcap_t);>

Return the file number of the network device opened with
B<Net::Pcap::open_live()>.

=back

=head2 Error handling

=over

=item B<Net::Pcap::geterr($pcap_t);>

Return an error message for the last error associated with the packet
capture device $pcap_t.

=item B<Net::Pcap::strerror($errno);>

Return a string describing error number $errno.

=item B<Net::Pcap::perror($pcap_t, $prefix);>

Print the text of the last error associated with descriptor $pcap_t on
standard error, prefixed by $prefix.

=back

=head2 New WinPcap functions

=item B<$error = Net::Pcap::createsrcstr(\$source, $type, $host, $port, $name, \$err);>

Accepts a set of strings ($host $name, $port), the $type of the source we
want to create, and it returns the complete $source string according to the
new format (e.g. 'rpcap://1.2.3.4/eth0').
Returns 0 if everything is fine, -1 if some errors occurred.

=item B<$error = Net::Pcap::parsesrcstr($source, \$type, \$host, \$port, \$name, \$err);>

Parses the C<$source> string and returns the pieces C<$type>, C<$host>, C<$port>,
C<$name> in which the source can be split.
Returns 0 if everything is fine, -1 if some errors occurred.

=item B<Net::Pcap::open($source, $snaplen, $flags, $read_timeout, \%auth, \$err);>

Open a generic source in order to capture / send (WinPcap only) traffic.
The Net::Pcap::open() replaces all the Net::Pcap::open_xxx() functions with
a single call.

Parameters:

=over 4

=item $source

The source name with the format prefix according to the new Source
Specification Syntax (See WinPcap doc).

On Linux systems with 2.2 or later kernels, a device argument of "any"
(i.e. rpcap://any) can be used to capture packets from all interfaces.

In order to makes the source syntax easier, please remember that:

=over 4

=item *

the adapters returned by the Net::Pcap::findalldevs_ex() can be used
immediately by the Net::Pcap::open()

=item *

in case the user wants to pass its own source string to the
Net::Pcap::open(), the Net::Pcap::createsrcstr() helps in creating the
correct source identifier.

=back

=item $snaplen

length of the packet that has to be retained. For each packet received by
the filter, only the first 'snaplen' bytes are stored in the buffer and
passed to the user application.

=item $flags

keeps several flags that can be needed for capturing packets. The allowed
flags are:

=over

=item C<OPENFLAG_PROMISCUOUS>

It defines if the adapter has to go in promiscuous mode.

=item C<OPENFLAG_DATATX_UDP>

It defines if the data transfer (in case of a remote capture) has to be done 
with UDP protocol.

=item C<OPENFLAG_NOCAPTURE_RPCAP>

It defines if the remote probe has to capture its own generated traffic.

=back

=item $read_timeout

read timeout in milliseconds. The read timeout is used to arrange that the
read not necessarily return immediately when a packet is seen, but that it
waits for some amount of time to allow more packets to arrive and to read
multiple packets from the OS kernel in one operation. Not all platforms
support a read timeout; on platforms that don't, the read timeout is
ignored.

=item %auth

a hash that keeps the information required to authenticate the user on a
remote machine. In case this is not a remote capture, this parameter can be
set to C<undef>.

The keys of the hash are C<type>, C<username>, C<password>.

Example:

  my %auth = ( type => RMTAUTH_PWD,
               username => 'TheUserName',
               password => 'ThePassWord'
             );

=item $err

if an error occurs the $err parameter will be set with an appropriate error
message. C<undef> otherwise.

=back

The function returns a packet capture descriptor for looking at packets on
the network. In case of problems, it returns C<undef> and the C<$err>
variable keeps the error message.

=back

=head2 Win32 specific functions

=over

=item B<Net::Pcap::setbuff($pcap_t, $dim)>

Sets the size of the kernel buffer associated with an adapter.
C<$dim> specifies the size of the buffer in bytes.
The return value is 0 when the call succeeds, -1 otherwise.

If an old buffer was already created with a previous call to
C<Net::Pcap::setbuff()>, it is deleted and its content is discarded.
C<Net::Pcap::open_live()> creates a 1 MByte buffer by default.

=item B<Net::Pcap::setmode($pcap_t, $mode)>

Sets the working mode of the interface C<$pcap_t> to C<$mode>.
Valid values for C<$mode> are C<MODE_CAPT> (default capture mode) and
C<MODE_STAT> (statistical mode).

=item B<Net::Pcap::setmintocopy($pcap_t, $size)>

Changes the minimum amount of data in the kernel buffer that causes a read
from the application to return (unless the timeout expires).

=item B<Net::Pcap::getevent($pcap_t)>

Returns the C<Win32::Event> object associated with the interface 
C<$pcap_t>. Can be used to wait until the driver's buffer contains some 
data without performing a read. See L<Win32::Event>.

=item B<Net::Pcap::sendpacket($pcap_t, $packet)>

Send a raw packet to the network. C<$pcap_t> is the interface that will be
used to send the packet, C<$packet> contains the data of the packet to send
(including the various protocol headers). The MAC CRC doesn't need to be
included, because it is transparently calculated and added by the network
interface driver. The return value is 0 if the packet is succesfully sent,
-1 otherwise.

=item B<$queue = Net::Pcap::sendqueue_alloc($memsize)>

This function allocates a send queue, i.e. a buffer containing a set of raw
packets that will be transmitted on the network with
C<Net::Pcap::sendqueue_transmit()>.

C<$memsize> is the size, in bytes, of the queue, therefore it determines 
the maximum amount of data that the queue will contain. This memory is 
deallocated  automatically when the queue ceases to exist.

=item B<Net::Pcap::sendqueue_queue($queue, \%hdr, $pkt)>

Adds a packet at the end of the send queue pointed by C<$queue>. The packet
header C<%hdr> has the same format as that passed to the
C<Net::Pcap::loop()> callback. C<$pkt> is a buffer with the data of
the packet.

The C<%hdr> header structure is the same used by WinPcap and libpcap to
store the packets in a file, therefore sending a capture file is
straightforward. 'Raw packet' means that the sending application will have
to include the protocol headers, since every packet is sent to the network
'as is'. The CRC of the packets needs not to be calculated, because it will
be transparently added by the network interface.

=item B<$nbytes = Net::Pcap::sendqueue_transmit($pcap_t, $queue, $sync)>

This function transmits the content of a queue to the wire. C<$pcap_t> is
the interface on which the packets will be sent, C<$queue> is to a
send_queue containing the packets to send, C<$sync> determines if the
send operation must be synchronized: if it is non-zero, the packets are
sent respecting the timestamps, otherwise they are sent as fast as
possible.

The return value is the amount of bytes actually sent. If it is smaller
than the size parameter, an error occurred during the send. The error can
be caused by a driver/adapter problem or by an inconsistent/bogus send
queue.

=head2 Export

By default, Net::Pcap exports no symbols into the callers namespace. The
following tags can be used to selectively import symbols into the main
namespace.

=over

=item :mode

  MODE_CAPT  MODE_STAT  MODE_MON

=item :src

  SRC_FILE  SRC_IFLOCAL  SRC_IFREMOTE

=item :openflag

  OPENFLAG_PROMISCUOUS  OPENFLAG_DATATX_UDP  OPENFLAG_NOCAPTURE_RPCAP

=item :samp

  SAMP_NOSAMP  SAMP_1_EVERY_N  SAMP_FIRST_AFTER_N_MS

=item :rpcap

  RMTAUTH_NULL  RMTAUTH_PWD

=back

=back

=head1 LIMITATIONS

The following limitations apply to this version of B<Net::Pcap>.

=over

=item *

At present, only one callback function and user data scalar can be
current at any time as they are both stored in global variables.

=back

=head1 EXAMPLES

See the 't' directory of the B<Net::Pcap> distribution for examples
on using this module.

=head1 COPYRIGHT

Copyright (c) 1999-2000 Tim Potter. All rights reserved. This program is
free software; you can redistribute it and/or modify it under the same
terms as Perl itself.

=head1 SEE ALSO

pcap(3), tcpdump(8)

The source code for libpcap is available from B<ftp://ftp.ee.lbl.gov/libpcap.tar.Z>

=head1 AUTHOR

Tim Potter E<lt>tpot@frungy.orgE<gt>

=cut
